/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _PREFDIALOG_H_
#define _PREFDIALOG_H_

#include<libglademm.h>
#include<gtkmm.h>
#include"pcaloptions.h"

class PCalMainWindow;

class PrefDialog : public Gtk::Window
{
    public:
		PrefDialog(BaseObjectType* cobject, 
                    const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
		virtual ~PrefDialog();
	
		/// This function should be used to show the preferences
        /// dialog. The parameter is the reference to the main window.
        virtual void present(PCalMainWindow *main_window);
	
	protected:
        Glib::RefPtr<Gnome::Glade::Xml> m_refGlade;
	private:      
        // the reference to the main window
        // the configuration manager and the 
        // plugins manager will be located there
        PCalMainWindow * pMainWindow;
        PCalOptions * calOptions;
    
        void getOptions();
        void storeOptions();
    
        // signal handlers
        void on_ok_click();
        void on_cancel_click();
        void on_defaults_click();
        // new option placeholder
        void select_cell_color(Gtk::Label* label, Gtk::EventBox* box, bool fg=false);
        void on_ColorTodayButton_click();
        void on_ColorPStartButton_click();
        void on_ColorPPStartButton_click();
        void on_ColorFertileButton_click();
        void on_ColorPFertileButton_click();
        void on_ColorPregnancyButton_click();
        void on_ColorBirthButton_click();
        void on_ColorPBirthButton_click();
        void on_FGColorTodayButton_click();
        void on_FGColorPStartButton_click();
        void on_FGColorPPStartButton_click();
        void on_FGColorFertileButton_click();
        void on_FGColorPFertileButton_click();
        void on_FGColorPregnancyButton_click();
        void on_FGColorBirthButton_click();
        void on_FGColorPBirthButton_click();
    
        // component references
        Gtk::RadioButton* startFromMondayRadiobutton;
        Gtk::RadioButton* startFromSundayRadiobutton;
        Gtk::ComboBox* toolbarStyleComboBox;
        // new option placeholder
        Gtk::Label*    colorTodayLabel;
        Gtk::EventBox* colorTodayEventBox;
        Gtk::Label*    colorPStartLabel;
        Gtk::EventBox* colorPStartEventBox;
        Gtk::Label*    colorPPStartLabel;
        Gtk::EventBox* colorPPStartEventBox;
        Gtk::Label*    colorFertileLabel;
        Gtk::EventBox* colorFertileEventBox;
        Gtk::Label*    colorPFertileLabel;
        Gtk::EventBox* colorPFertileEventBox;
        Gtk::Label*    colorPregnancyLabel;
        Gtk::EventBox* colorPregnancyEventBox;
        Gtk::Label*    colorBirthLabel;
        Gtk::EventBox* colorBirthEventBox;
        Gtk::Label*    colorPBirthLabel;
        Gtk::EventBox* colorPBirthEventBox;
        Gtk::SpinButton* reserveSpinbutton;
        Gtk::SpinButton* badFeelStartSpinbutton;
        Gtk::SpinButton* badFeelEndSpinbutton;
        Gtk::SpinButton* regCycleMinSpinbutton;
        Gtk::SpinButton* regCycleMaxSpinbutton;
        Gtk::Notebook* prefNotebook;
        
        // dialog action area
        Gtk::HButtonBox* prefButtonBox;
        Gtk::Button* prefOKButton;

};

#endif	//_PREFDIALOG_H_
