/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include"viewmodel.h"
#include<ctime>
#include<cstdio>
#include<iostream>
#include"globals.h"
#include"calcellrenderer.h"
#include"utils.h"

MonthCalendarModel::MonthCalendarModel(MonthCalendarColumns * calendarColumns)
:Gtk::ListStore(*calendarColumns),
columns(calendarColumns)
{
    // get the current date
    todayDate = PCal::getTodayDate();
}

void MonthCalendarModel::initialize(PCalMainWindow * mv)
{
    current_date.set_dmy(1,Glib::Date::JANUARY,1900);
    mainWindow = mv;
    mainWindow->model_today_date = &todayDate;
    calOptions = mainWindow->calOptions;
    // GET THE WEEK DAY NAMES FROM LOCALE
    // I know this is Monday
    Glib::Date tempDate(28,Glib::Date::JUNE,2004);
    if(calOptions->firstDay == Glib::Date::SUNDAY)
        tempDate.subtract_days(1);

    for(int i=0;i<7;i++) {
        CalCellRenderer* pRenderer = Gtk::manage( new CalCellRenderer() );
        
        int cols_count = mainWindow->calendarTreeView->append_column(tempDate.format_string("%a"), *pRenderer);
        Gtk::TreeViewColumn* pColumn = mainWindow->calendarTreeView->get_column(cols_count-1);
        if(pColumn)
        {
            pColumn->add_attribute(pRenderer->property_text(), columns->day[i]);
            pColumn->add_attribute(pRenderer->property_background_gdk(), columns->bgColors[i]);
            pColumn->add_attribute(pRenderer->property_foreground_gdk(), columns->fgColors[i]);
            pColumn->add_attribute(pRenderer->property_focused(), columns->focused[i]);
        }
        tempDate.add_days(1);
    }
    mainWindow->calendarTreeView->columns_autosize();
    
    // rebuild for the today date
    jumpTo(todayDate);
} 

void MonthCalendarModel::jumpTo(const Glib::Date date, const bool force_refresh) {
    Glib::Date tmpDate = date;
    // was the month changed?
    if(force_refresh || tmpDate.get_month()!=current_date.get_month() ||
       tmpDate.get_year()!=current_date.get_year()) 
    {
        clear();
        tmpDate.set_day(1);
        bool day1Offset = false;
        // prevent crash if we are going to display January of the year 1 and week is starting from Sunday
        if((tmpDate.get_year() == 1) && (tmpDate.get_month() == 1) && (calOptions->firstDay == Glib::Date::SUNDAY)) {
         day1Offset = true;
       } else {
          while(tmpDate.get_weekday()!=calOptions->firstDay) {
              tmpDate.subtract_days(1);
          }
        }
        // there are no more than 6 lines per month
        for(int i=0;i<6;i++)
        {
            Gtk::TreeModel::Row row = *append();
            for(int j=0;j<7;j++) {
                row[columns->date[j]] = tmpDate;
                if(day1Offset) {
                    day1Offset = false;
                    row[columns->day[j]] = "-";
                    row[columns->bgColors[j]] = calOptions->inactiveMonthBGColor;
                    row[columns->fgColors[j]] = calOptions->inactiveMonthFGColor;
                } else {
                    row[columns->day[j]] = tmpDate.format_string("%e");
                    if(tmpDate == todayDate) {                    
                        row[columns->bgColors[j]] = calOptions->todayBGColor;
                        row[columns->fgColors[j]] = calOptions->todayFGColor;                    
                    } else
                    if(tmpDate.get_month()==date.get_month()) {
                        PCalDayInfo dayInfo;
                        dayInfo = mainWindow->engine->getDayInfo(tmpDate);
                        
                        if(dayInfo.day_num == -1) {
                            row[columns->bgColors[j]] = calOptions->undefinedMonthBGColor;
                            row[columns->fgColors[j]] = calOptions->undefinedMonthFGColor;
                        } else
                        if(dayInfo.birth) {
                            if(dayInfo.estimate){
                                row[columns->bgColors[j]] = calOptions->pbirthBGColor;
                                row[columns->fgColors[j]] = calOptions->pbirthFGColor;
                            } else {
                                row[columns->bgColors[j]] = calOptions->birthBGColor;
                                row[columns->fgColors[j]] = calOptions->birthFGColor;
                            }
                        } else
                        if(dayInfo.pregnancy) {
                            row[columns->bgColors[j]] = calOptions->pregBGColor;
                            row[columns->fgColors[j]] = calOptions->pregFGColor;
                        } else {
                            if(dayInfo.day_num == 0) {
                                if(dayInfo.estimate){
                                    row[columns->bgColors[j]] = calOptions->pday1BGColor;
                                    row[columns->fgColors[j]] = calOptions->pday1FGColor;
                                } else {
                                    row[columns->bgColors[j]] = calOptions->day1BGColor;
                                    row[columns->fgColors[j]] = calOptions->day1FGColor;
                                }
                            } else 
                            if(dayInfo.fertile)
                            {
                                if(dayInfo.estimate){
                                    row[columns->bgColors[j]] = calOptions->pfertBGColor;
                                    row[columns->fgColors[j]] = calOptions->pfertFGColor;
                                } else {
                                    row[columns->bgColors[j]] = calOptions->fertBGColor;
                                    row[columns->fgColors[j]] = calOptions->fertFGColor;
                                }                        
                            }
                        }
                    } else {
                        row[columns->bgColors[j]] = calOptions->inactiveMonthBGColor;
                        row[columns->fgColors[j]] = calOptions->inactiveMonthFGColor;
                    }
                    row[columns->focused[j]] = (tmpDate == date);
                    tmpDate.add_days(1);
                }
            }
            if(tmpDate.get_month()!=date.get_month()) break;
        }
        mainWindow->monthLabel->set_label("<b>"+date.format_string("%B") + "</b>");
        mainWindow->yearLabel->set_label("<b>"+date.format_string("%Y") + "</b>");
        mainWindow->cal_current_date = date;
    } // if month changed
    
    current_date = date;    
}

void MonthCalendarModel::setToday(const Glib::Date date) {
    todayDate = date;
}

void MonthCalendarModel::refresh()
{
    jumpTo(current_date, true);
}

bool MonthCalendarModel::updateRowSelections(const Gtk::TreeModel::Path& path, const Gtk::TreeModel::iterator& iter)
{
    // it is supposed to have a single selection only
    for(int i=0; i<7; i++) {
        (*iter)[columns->focused[i]] = 
            ((*iter)[columns->date[i]] == current_date);
    }
    return false;
}

void MonthCalendarModel::updateSelection()
{
    foreach(sigc::mem_fun(*this, &MonthCalendarModel::updateRowSelections));
}
